@@ -1,29 +1,30 @@
-# @(#)$Id: Result.pm 285 2011-07-11 12:40:49Z pjf $
+# @(#)$Id: Result.pm 321 2011-11-30 00:01:49Z pjf $
 
 package File::DataClass::Result;
 
 use strict;
 use namespace::clean -except => 'meta';
-use version; our $VERSION = qv( sprintf '0.6.%d', q$Rev: 285 $ =~ /\d+/gmx );
+use version; our $VERSION = qv( sprintf '0.7.%d', q$Rev: 321 $ =~ /\d+/gmx );
 
 use Moose;
 
 has 'name'       => is => 'rw', isa => 'Str',    required => 1;
-has '_resultset' => is => 'ro', isa => 'Object', required => 1;
+has '_resultset' => is => 'ro', isa => 'Object', required => 1,
+   handles       => { _path    => q(path), _source => q(source),
+                      _storage => q(storage) };
 
 sub BUILD {
    my ($self, $args) = @_; my $class = blessed $self;
 
-   my %types = ( qw(SCALAR Maybe[Str]
-                    ARRAY  Maybe[ArrayRef]
+   my %types = ( qw(SCALAR Maybe[Str] ARRAY  Maybe[ArrayRef]
                     HASH   Maybe[HashRef]) );
 
-   for (@{ $self->_resultset->source->attributes }) {
-      my $type = ref $args->{ $_ } || q(SCALAR);
+   for (@{ $self->_source->attributes }) {
+      my $type = ref $self->_source->defaults->{ $_ } || ref $args->{ $_ };
 
       $class->meta->has_attribute( $_ )
          or $class->meta->add_attribute
-            ( $_ => ( is => 'rw', isa => $types{ $type } ) );
+            ( $_ => ( is => 'rw', isa => $types{ $type || q(SCALAR) } ) );
 
       defined $args->{ $_ } and $self->$_( $args->{ $_ } );
    }
@@ -32,25 +33,15 @@ sub BUILD {
 }
 
 sub delete {
-   my $self = shift; return $self->_storage->delete( $self->_path, $self );
+   return $_[ 0 ]->_storage->delete( $_[ 0 ]->_path, $_[ 0 ] );
 }
 
 sub insert {
-   my $self = shift; return $self->_storage->insert( $self->_path, $self );
+   return $_[ 0 ]->_storage->insert( $_[ 0 ]->_path, $_[ 0 ] );
 }
 
 sub update {
-   my $self = shift; return $self->_storage->update( $self->_path, $self );
-}
-
-# Private methods
-
-sub _path {
-   return shift->_resultset->path;
-}
-
-sub _storage {
-   return shift->_resultset->storage;
+   return $_[ 0 ]->_storage->update( $_[ 0 ]->_path, $_[ 0 ] );
 }
 
 no Moose;
@@ -67,7 +58,7 @@ File::DataClass::Result - Result object definition
 
 =head1 Version
 
-0.6.$Revision: 285 $
+0.7.$Revision: 321 $
 
 =head1 Synopsis
 
