@@ -1,18 +1,18 @@
-# @(#)$Id: Storage.pm 321 2011-11-30 00:01:49Z pjf $
+# @(#)$Id: Storage.pm 324 2011-12-01 03:42:25Z pjf $
 
 package File::DataClass::Storage;
 
 use strict;
 use namespace::autoclean;
-use version; our $VERSION = qv( sprintf '0.7.%d', q$Rev: 321 $ =~ /\d+/gmx );
+use version; our $VERSION = qv( sprintf '0.7.%d', q$Rev: 324 $ =~ /\d+/gmx );
 
+use Moose;
 use Class::Null;
 use English qw(-no_match_vars);
 use File::Copy;
 use File::DataClass::Constants;
 use File::DataClass::HashMerge;
 use Hash::Merge qw(merge);
-use Moose;
 use Try::Tiny;
 
 extends qw(File::DataClass);
@@ -159,9 +159,7 @@ sub _create_or_update {
 
    try {
       my $filter = sub { __get_src_attributes( $condition, $_[ 0 ] ) };
-      my $name   = $result->name;
-
-      $data->{ $element } ||= {};
+      my $name   = $result->name; $data->{ $element } ||= {};
 
       not $updating and exists $data->{ $element }->{ $name }
          and $self->throw( error => 'File [_1] element [_2] already exists',
@@ -276,7 +274,7 @@ File::DataClass::Storage - Storage base class
 
 =head1 Version
 
-0.7.$Revision: 321 $
+0.7.$Revision: 324 $
 
 =head1 Synopsis
 
